import re
from rich.console import Console
from rich.text import Text
from rich.panel import Panel
from typing import NamedTuple, List
from html import unescape
from rich.console import Group

def extract_text(html):
    res = re.search(r'>(.*)<', html)
    if res is not None:
        return res.group(1)


def split_authors(authors):

    return [extract_text(author) for author in authors[0]['name'].split(',') if extract_text(author) is not None]


class Article(NamedTuple):
    title: str
    abstract: str
    authors: List[str]
    link: str

    @classmethod
    def from_entry(cls, entry: dict) -> 'Article':
        return cls(
            entry['title'],
            entry['summary'][3:-4].replace('\n', ' '),
            split_authors(entry['authors']),
            entry['link']
        )

    def __rich__(self):

        if len(self.authors) > 10:
            author_list = self.authors[:10] + ["et al."]
        else:
            author_list = self.authors

        # make a string with all the authors separated by commas
        authors = ", ".join(author_list)
        # decode numerical unicode characters in the authors string
        authors = unescape(authors)

        panel_group = Group(
            Panel(Text(self.title, justify="center"), style="bold red"),
            Panel(Text(authors, justify="center"), style="italic green", title='Authors'),
            Panel(Text(self.abstract, justify="full")),
            Panel(Text(self.link, justify="center"), style="bold yellow")
        )

        return panel_group#text

    # make a method to print the article in a rich way as done in today.py
    def to_console(self, console: Console):
        console.rule(self.title, style="bold red")
        # this should be done in the feed class
        # if there are more than 10 authors, keep only the first 10 and replace the rest with "et al."
        if len(self.authors) > 10:
            author_list = self.authors[:10] + ["et al."]
        else:
            author_list = self.authors

        # make a string with all the authors separated by commas
        authors = ", ".join(author_list)
        # decode numerical unicode characters in the authors string
        authors = unescape(authors)

        console.print(f"{authors}", style="italic green")
        console.print(self.abstract)
        console.print()


import feedparser
from .article import Article


class ArxivFeed:

    def __init__(self, feed_url='https://arxiv.org/rss/astro-ph'):
        self.feed_url = feed_url
        self.feed = feedparser.parse(self.feed_url)

        articles = []

        for entry in self.feed['entries']:
            articles.append(Article.from_entry(entry))
        self.articles = articles


today: ArxivFeed = ArxivFeed()



from textual.app import App
from textual.widgets import Footer, Header
from .list import ListViewUo
from arxivAssistant.feed import ArxivFeed
from textual.reactive import Reactive
from textual.widgets import ScrollView
from textual.widgets import Button, ButtonPressed
from rich.panel import Panel


class ArticleName(Button):

    mouse_over = Reactive(False)

    def __init__(self, text: str):
        super().__init__(text)
        self.text = text

    def render(self) -> Panel:
        return Panel(self.text, style=("on red" if self.mouse_over else ""))

    def on_enter(self) -> None:
        self.mouse_over = True

    def on_leave(self) -> None:
        self.mouse_over = False


class ArxivApp(App):

    async def on_mount(self) -> None:
        today = ArxivFeed()
        self.body = ScrollView()
        self.articles = {article.title:article for article in today.articles}

        await self.view.dock(Header(), edge="top")
        await self.view.dock(Footer(), edge="bottom")
        await self.view.dock(
            ListViewUo([ArticleName(article.title) for article in today.articles]), edge="left", size=60)
        await self.view.dock(self.body, edge="right")

    async def on_load(self) -> None:
        """Bind keys here."""
        await self.bind("q", "quit", "Quit")

    async def handle_button_pressed(self, message: ButtonPressed) -> None:
        """A message sent by the button widget"""

        assert isinstance(message.sender, Button)
        button_name = message.sender.name

        await self.body.update(self.articles[button_name])



"""
Stolen from https://github.com/Cvaniak/TextualListViewUnofficial.git for lightweight compatibility.
"""


from __future__ import annotations

from rich.console import RenderableType

from textual.geometry import Size, SpacingDimensions
from textual.layouts.vertical import VerticalLayout
from textual.views._window_view import WindowChange
from textual.reactive import Reactive
from textual.widgets import Placeholder
from textual.widgets import ScrollView
from textual.widget import Widget
from textual.view import View
from textual.events import Message
from textual import messages
from textual.widgets import Static
from textual.app import App
from textual import events

from typing import List


class MultipleWidgetsWindowView(View, layout=VerticalLayout):
    layout: Reactive[VerticalLayout]

    def __init__(
        self,
        widgets: List[RenderableType | Widget] | None = None,
        *,
        auto_width: bool = False,
        gutter: SpacingDimensions = (0, 0),
        name: str | None = None
    ) -> None:
        layout = VerticalLayout(gutter=gutter, auto_width=auto_width)
        super().__init__(name=name, layout=layout)
        self.set_widgets(widgets)
        for widget in self._widgets:
            layout.add(widget)

    async def update(self, widgets: List[Widget | RenderableType] | None) -> None:
        layout = self.layout
        assert isinstance(layout, VerticalLayout)
        self.set_widgets(widgets)
        await self.arrange_widgets()

    def set_widgets(self, widgets: List[Widget | RenderableType] | None) -> None:
        if widgets is None:
            return
        self._widgets: List[Static | Widget] = [
            w if isinstance(w, Widget) else Static(w) for w in widgets
        ]

    async def arrange_widgets(self):
        self.layout.clear()
        for widget in self._widgets:
            self.layout.add(widget)
        self.layout.require_update()
        self.refresh(layout=True)
        await self.emit(WindowChange(self))

    async def handle_update(self, message: messages.Update) -> None:
        message.prevent_default()
        await self.emit(WindowChange(self))

    async def handle_layout(self, message: messages.Layout) -> None:
        self.log("TRANSLATING layout")
        self.layout.require_update()
        message.stop()
        self.refresh()

    async def watch_virtual_size(self, size: Size) -> None:
        await self.emit(WindowChange(self))

    async def watch_scroll_x(self, value: int) -> None:
        self.layout.require_update()
        self.refresh()

    async def watch_scroll_y(self, value: int) -> None:
        self.layout.require_update()
        self.refresh()

    async def on_resize(self, event: events.Resize) -> None:
        await self.emit(WindowChange(self))

    async def add_widget(
        self, widget: Widget | RenderableType, index: int | None = None
    ):
        if index is None:
            index = len(self._widgets)
        index = min(len(self._widgets), max(0, index))
        self._widgets.insert(
            index, widget if isinstance(widget, Widget) else Static(widget)
        )
        await self.arrange_widgets()

    async def remove_widget_by_index(self, index: int = 0):
        if not self._widgets:
            return
        index = min(len(self._widgets)-1, max(0, index))
        self._widgets.pop(index)
        await self.arrange_widgets()

    async def remove_widget(self, widget: Widget):
        if not self._widgets:
            return
        self._widgets.remove(widget)
        await self.arrange_widgets()


class ListViewUo(ScrollView):
    def __init__(
        self, widgets: List[Widget | RenderableType] | None = None, *args, **kwargs
    ) -> None:
        super().__init__(*args, **kwargs)
        self.widgets_list = widgets
        self.window = MultipleWidgetsWindowView(self.widgets_list)

    def refresh_all(self):
        self.window.layout.require_update()
        self.layout.require_update()
        self.window.refresh()
        self.vscroll.refresh()
        self.hscroll.refresh()

    async def add_widget(self, widget:Widget, index:int|None = None):
        await self.window.add_widget(widget, index)
        self.refresh_all()

    async def remove_widget_by_index(self, index: int=0):
        await self.window.remove_widget_by_index(index)
        self.refresh_all()

    async def remove_widget(self, widget: Widget):
        await self.window.remove_widget(widget)
        self.refresh_all()


if __name__ == "__main__":
    from textual.widgets import Footer

    class DeleteStatus(Message):
        def __init__(self, sender: Widget):
            super().__init__(sender)
            self.to_delete = sender

    class DeletablePlaceholder(Placeholder):
        async def on_click(self, event: events.Click) -> None:
            await self.emit(DeleteStatus(self))

    class TestListView(App):
        async def action_add(self) -> None:
            await self.list_view.add_widget(DeletablePlaceholder(height=10))
            self.refresh()

        async def action_add_index_2(self) -> None:
            await self.list_view.add_widget(DeletablePlaceholder(height=10), index=2)
            self.refresh()

        async def action_remove(self) -> None:
            await self.list_view.remove_widget_by_index()
            self.refresh()

        async def action_remove_index_2(self) -> None:
            await self.list_view.remove_widget_by_index(index=2)
            self.refresh()

        async def on_load(self, _: events.Load) -> None:
            await self.bind("a", "add()", "Add Widget")
            await self.bind("s", "add_index_2()", "Add Widget in index 2")
            await self.bind("r", "remove()", "Remove Widget")
            await self.bind("e", "remove_index_2()", "Remove Widget in index 2")
            await self.bind("Click Widget", "_()", "To delete it")

        async def on_mount(self, event: events.Mount) -> None:
            self.list_view = ListViewUo(
                [DeletablePlaceholder(height=10) for _ in range(7)]
            )

            await self.view.dock(Footer(), edge="bottom")
            await self.view.dock(self.list_view)

        async def handle_delete_status(self, message: DeleteStatus):
            await self.list_view.remove_widget(message.to_delete)

    TestListView.run()


import sys
from setuptools import setup, find_packages

__version__ = '0.3.1'

# READ README.md for long description on PyPi.
try:
    long_description = open("README.md", encoding="utf-8").read()
except Exception as e:
    sys.stderr.write("Failed to read README.md:\n  {}\n".format(e))
    sys.stderr.flush()
    long_description = ""

setup(
    name='arxivAssistant',
    version=__version__,
    python_requires='>=3.9',
    description='Read daily arxiv feed with console',
    author='Simon Dupourqué',
    author_email='sdupourque@irap.omp.eu',
    package_dir={'': "src"},
    packages=find_packages("src"),
    install_requires=[
        'feedparser',
        'rich',
        'textual'
    ],
    url='https://github.com/renecotyfanboy/arxivAssistant',
    license='MIT',
    zip_safe=False,
    entry_points={
            'console_scripts': [
                'arxiv-today=arxivAssistant.scripts.today:main'
            ],
        },
    long_description=long_description,
    long_description_content_type="text/markdown",
)